Course:     Heuristic Problem Solving Fall 2021
            CSCI-GA.2965-001
            Dennis Shasha
            Graduate Division
            Computer Science

Location:   Warren Weaver Hall (251 Mercer Street)

Title:      Dating Game

Architects: Adam Putzer (alp593@nyu.edu) and Guanqun Yang (gy2025@nyu.edu)

## About the Game

Matchmaker M is trying to find the `ideal` candidate C for player P.

P indicates initial preferences for N attributes of C but tries to be `hard to please` by seeing M's Cs and modifying their initial preference within a set of constraints after each proposal from M.

The game involves M specifying a series of candidate Cs for P.  The Server will report to M how much P likes those dates (score between -1 and 1, where 1 is good and -1 is very bad).  P's criteria for liking a date or not depends on the weights P gives to various attributes -- e.g. literary knowledge, ability to solve puzzles, and others.  The weights may be positive or negative ranging from -1 to 1 and specified as decimal values having at most two digits to the right of the decimal point, e.g. 0.13 but not 0.134.

P must respond with updated preferences after each C is proposed by M.  In each response, P may modify 5% of their attribute weights (P may choose which ones) by 20% each with respect to the original weights.  For example, if a chosen attribute has a weight of 0.4 in P's original criteria, then P can modify it to any value between 0.4 - (0.2*0.4) and 0.4 + (0.2*0.4).  P does these modifications after seeing M's candidates.  (Modifications are always with respect to P's original choice of weights.)  Also, in both the original setting of weights and after all modifications, P must ensure that the sum of P's positive weights (there must be at least one such weight) must be 1 and the sum of the negative weights (there must be at least one such weight) must be -1.

A candidate C has values for each of N (bigger than 1 and not more than 200) attributes -- each value lies between 0 and 1 inclusive and must have four or fewer digits of precision (without this constraint, there is a way to discover the sign of the weight that P ascribes to each attribute).  Given the weights P has assigned to each attribute w1, ..., wn and the values C has for each attribute v1, ..., vn, the score P gives to C is simply the dot product of the two vectors.  No candidate should get a score below -1 or above 1, but the ideal candidate will consist solely of 1s and 0s.

## Compiling and running a game server with connections to Dater and Matchmaker clients.

Compile the game server with a java 7 or newer compiler.

`javac DatingGameServer.java`

Command to run a server to host an instance of the dating game with N candidate attributes.

`java DatingGameServer {N}`

### Dater  P

The Dater will be communicating with the game server over a socket at **port `20000`**.

Sample dater clients are available in java, python, and c++ that send a dater profile starting with "1:-1" followed by ":0" repeated {N - 2} times based on the value of N communicated from the server.  The same weights will be used for each of the Dater's 20 turns.

### Matchmaker  M

The Matchmaker will be communicating with the game server over a socket at **port `20001`**.

Sample Matchmaker clients are available in java, python, and c++ that send a candidate with valid random values for the N attributes based on the value of N implied from the server.  The sample clients send a new random candidate to the server for each of the 20 turns.

### Game Initialization

Dater (P) sends the message `Person` to a running server.

The server sends the message of `N:{N}` (N will be an integer bigger than 1 and less than 200) to P.

P sends an initial dating profile with weights for the N attributes, delimited with colons, to the server.  A valid N=5 example is `0.25:0.10:0.65:-0.40:-0.60`.

The Server will verify that the constraints are met otherwise disconnect in a loss for P.

Matchmaker (M) sends the message `Matchmaker` to the server.

The Server generates a candidate C with values 0 or 1 (0.5 probability of each) for each of the N attributes and calculates a score for C using P's dating profile.  The server sends the weights and score, delimited with colons to M.  A valid `N=5` example scoring `0.35` is `0:1:1:1:0:0.35`.

The Server repeates the previous step 19 additional times.

### Game Play

The game consists of up to 20 rounds where;
* M sends a candidate C to S
  * Valid example message for C: `0.25:0.10:0.65:0.40:0.60`
* S verifies C meets the constraints
  * Otherwise M loses
* S sends C to P
  * Valid example message for C: `0.25:0.10:0.65:0.40:0.60`
* P sends a modified dating profile to S
  * Valid example message for profile: `0.27:0.08:0.65:-0.40:-0.60`
* S verifies the modified profile meets the constraints
  * Otherwise P loses
* S checks if the score using the original profile's weights is 1
  * If so, the game ends at this round
* S sends M a message of the form `SCORE:{S}` where S is the score from the modified dating profile with exactly four digits of precision.

Throughout the game both M and P have 2 minutes, accumulated whenever S is waiting on a message from the respective client, for decision making.  A client using longer than 2 minutes for decision making results in that client losing the game.

### Game End

The total `Value` for M is the lexicographic string `{highest-score}, {round-of-highest-score}`.

## CSCI-GA.2965-001 Game Night: Tuesday November 16th, 2021

Our class is partitioned into teams and each team is expected to send the architects (before game night) two client programs; one that acts as a Dater and one that acts as a Matchmaker.

Two teams' submissions, X and Y, will be pairwise evaluated by running two instances of the Dating Game;
* X's M against Y's P
* Y's M against X's P

X beats Y if X's M obtains a Value against Y's P which is greater than the Value from Y's M against X's P.

### Results

| Team          | Result                               |
|---------------|-------------------------------------:|
| omniheurist   |                                      |
| TeamZ         |                                      |
| BlackBox      |                                      |
| Omicron       |                                      |
| RedScarf      |                                      |
| ys            |                                      |
| coco          |                                      |
| space marines | Delayed entry                        |
