import java.io.File;

import java.util.ArrayList;

public class ScoreAll {
	public static void main(String[] args) {
		ArrayList<String> teamNames = new ArrayList<String>();

		//get all directories which should correspond to teams
		String binDir = System.getProperty("user.dir");
		String startingDir = binDir.replace("/scoreSrc", "");
		String scriptName = "testshuf.sh";
		String outFileName = "outShuf.dat";
		File[] teamDirs = new File(startingDir).listFiles();
		for (File teamDir : teamDirs) {
			if (teamDir.isDirectory() && !teamDir.getName().equals("scoreSrc")) {
				teamNames.add(teamDir.getName());
			}
		}

		//Initalize Score System
		String cityInfoFile = args[0];
		ScoreGen scorer = new ScoreGen(cityInfoFile);
		Standings results = new Standings();
		double timeLimitMinutes = 2;
		int timeLimitMilliseconds = (int) (timeLimitMinutes * 60000);

		String execFileLoc, outFileLoc, cmd;
		long startTime, endTime, timeTaken;
		double score = 0.0;
		boolean allVisit = false;
		boolean underLimit = false;
		timeTaken = -1;

		//loop through all teams
		for (String team : teamNames) {
			System.out.print("Working on " + team + "...\n");
			//default values and clear scorer path
			timeTaken = -1;
			allVisit = false;
			underLimit = false;
			score = 0.0;
			scorer.clearVisits();

			//files and command info
			execFileLoc = startingDir +"/" + team + "/" + scriptName;
			outFileLoc = startingDir +"/" + team + "/" + outFileName;
			cmd = execFileLoc + " " + outFileLoc + " " + cityInfoFile;

			//run user program
			startTime = System.currentTimeMillis();
			endTime = 0;
			try{
				Process p = Runtime.getRuntime().exec(cmd);
				int exitCode = p.waitFor();
				endTime = System.currentTimeMillis();
			}
			catch (Exception e) {e.printStackTrace();}
			timeTaken = endTime -startTime;

			//score output
			score = scorer.scorePathFile(outFileLoc);
			allVisit = scorer.wereAllVisited();
			underLimit = timeTaken<=timeLimitMilliseconds;

			TeamResult teamResult = new TeamResult(
				team, score, timeTaken, allVisit, underLimit);

			results.addTeam(teamResult);

			//System.out.println(team + " " + timeTaken + " " + score + " " + allVisit);
		}

		results.printOutStandings();

		// String cityInfoFile = "travelingtest.txt";
		// String pathFile = "testpath.txt";
		// ScoreGen scorer = new ScoreGen(cityInfoFile);
		// double output = scorer.scorePathFile(pathFile);
		// boolean vistBool = scorer.wereAllVisited();
		// System.out.println(output);
		// System.out.println(vistBool);
	}
}