#!/usr/bin/env python
import sys, os, re, fileinput

# The usage is simple. You can just pass the original data file
# and a result file in the format the professor specified:
#   $ ./validator.py datafile resultfile
# Ambulance: 1: (45,32), 5: (84,26,95) total: 92
# Ambulance: 1: (45,32), 6: (64,30,74), 11: (56,26,80) total: 53
# Ambulance: 1: (45,32), 42: (29,42,67) total: 54
# Ambulance: 2: (59,68), 8: (68,57,52) total: 42
# Ambulance: 2: (59,68), 14: (66,47,58) total: 58
# Ambulance: 2: (59,68), 19: (64,75,47), 33: (69,72,37) total: 37
# By Yusuke Shinyama

def timetake(a,b):
 return abs(a.x-b.x)+abs(a.y-b.y)

PID = 0
class Person:
  def __init__(self, x, y, st):
    global PID
    PID += 1
    self.pid = PID
    self.x = x
    self.y = y
    self.st = st
    self.rescued = False
    return
  def __repr__(self):
    return '%d: (%d,%d,%d)' % (self.pid, self.x, self.y, self.st)

HID = 0
class Hospital:
  def __init__(self, x, y, namb):
    global HID
    HID += 1
    self.hid = HID
    self.x = x
    self.y = y
    self.ambtime = { 0:namb }
  def __repr__(self):
    return '%d: (%d,%d)' % (self.hid, self.x, self.y)
  def decamb(self, t):
    self.ambtime[t] -= 1
    if self.ambtime[t] == 0: del self.ambtime[t]
    return
  def incamb(self, t):
    if t not in self.ambtime: self.ambtime[t] = 0
    self.ambtime[t] += 1
    return
  def takeperson(self, p1, p2):
    t0 = min(self.ambtime.keys())
    if p1.rescued:
      raise 'Person already rescued: %s' % p1
    if p2 == None:
      t = timetake(self,p1)*2+2
      if p1.st < t0+t:
        raise 'Person must die: %s' % p1
    else:
      if p2.rescued:
        raise 'Person already rescued: %s' % p2
      t = timetake(self,p1)+1+timetake(p1,p2)+1+timetake(p2,self)+1
      if p1.st < t0+t:
        raise 'Person must die: %s' % p1
      if p2.st < t0+t:
        raise 'Person must die: %s' % p2
    self.decamb(t0)
    self.incamb(t0+t)
    p1.rescued = True
    if p2:
      p2.rescued = True
      print 'Rescued:', p1, 'and', p2
    else:
      print 'Rescued:', p1
    return

def readdata(fname):
  print >>sys.stderr, 'Reading data:', fname
  persons = []
  hospitals = []
  mode = 0
  for line in file(fname):
    line = line.strip().lower()
    if line.startswith("person") or line.startswith("people"):
      mode = 1
    elif line.startswith("hospital"):
      mode = 2
    elif line:
      (a,b,c) = map(int, line.split(","))
      if mode == 1:
        persons.append(Person(a,b,c))
      elif mode == 2:
        hospitals.append(Hospital(a,b,c))
  return (persons, hospitals)

def readout(persons, hospitals):
  print >>sys.stderr, 'Reading results...'
  p1 = re.compile(r'(\d+\s*:\s*\(\s*\d+\s*,\s*\d+(\s*,\s*\d+)?\s*\))')
  p2 = re.compile(r'(\d+)\s*:\s*\(\s*(\d+)\s*,\s*(\d+)\s*\)')
  p3 = re.compile(r'(\d+)\s*:\s*\(\s*(\d+)\s*,\s*(\d+)\s*,\s*(\d+)\s*\)')
  score = 0
  for line in fileinput.input():
    line = line.strip()
    if not line: continue
    if not line.lower().startswith('ambulance'):
      print 'Ignored: %r' % line
      continue
    hos = None
    tookpersons = []
    for (i,(w,z)) in enumerate(p1.findall(line)):
      m = p2.match(w)
      if m:
        if i != 0: raise 'Syntax error, expected "n:(x,y)": %r' % line
        (a,b,c) = map(int, m.groups())
        hos = hospitals[a-1]
        if hos.x != b or hos.y != c: raise 'Hospital mismatch: %s != %d:%s' % (hos, a,(b,c))
      else:
        m = p3.match(w)
        if not m: raise 'Syntax error, expected "n:(x,y,t)": %r' % line
        if i == 0: raise 'Syntax error, specify a hospital first: %r' % line
        (a,b,c,d) = map(int, m.groups())
        per = persons[a-1]
        if per.x != b or per.y != c or per.st != d: raise 'Person mismatch: %s != %d:%s' % (per, a,(b,c,d))
        tookpersons.append(per)
    if not hos or not tookpersons:
      print 'Ignored: %r' % line
      continue
    if len(tookpersons) == 1:
      hos.takeperson(tookpersons[0], None)
    elif len(tookpersons) == 2:
      hos.takeperson(tookpersons[0], tookpersons[1])
    else:
      raise 'Cannot take more than two people: %r' % line
    score += len(tookpersons)
  print 'Total:', score
  return

# main
if __name__ == "__main__":
  if len(sys.argv) < 2:
    print 'usage: validator.py datafile [resultfile]'
    sys.exit(2)
  (persons, hospitals) = readdata(sys.argv[1])
  del sys.argv[1]
  readout(persons, hospitals)

